@@ -24,16 +24,20 @@
     "<U",
     "|b1"
   )
+  r_base_types <- c("integer", "double", "character", "logical")
+
   if (data_type %notin% supported_types) {
+    if (data_type %notin% r_base_types) {
+      stop(
+        "Currently only able to write integer, double, character and logical arrays"
+      )
+    }
     data_type <- switch(
       data_type,
       "integer" = "<i4",
       "double" = "<f8",
       "character" = "|S",
-      "logical" = "|b1",
-      stop(
-        "Currently only able to write integer, double, character and logical arrays"
-      )
+      "logical" = "|b1"
     )
   }
 
@@ -219,6 +223,10 @@ write_zarr_array <- function(
     version_from = 2,
     version_to = 3
   )
+  metadata_v3$configured_encoders <- .configure_codecs(
+    codecs = metadata_v3$codecs,
+    operation = "encode"
+  )
 
   chunk_indices <- .generate_chunk_indices(
     x_dim = dim(x),
@@ -228,24 +236,29 @@ write_zarr_array <- function(
     chunk_indices,
     metadata_v3
   )
+  chunk_paths <- paste0(path, chunk_names)
+  chunk_indices <- apply(chunk_indices, 1, identity, simplify = FALSE)
 
-  same_type_lower_bytesize <- metadata$dtype %in% c("|i1", "<i2", "<f4")
+  same_type_lower_bytesize <- metadata_v3$data_type %in%
+    c("int8", "int16", "float32")
   lower_bytesize_type <- storage.mode(x) == "double" &&
-    metadata$dtype == "<i4"
+    metadata_v3$data_type == "float32"
 
   can_overflow <- same_type_lower_bytesize || lower_bytesize_type
   if (can_overflow) {
-    x <- .truncate_overflow(x, metadata$dtype)
+    x <- .truncate_overflow(x, metadata_v3$datatype$nbytes)
   }
 
   ## iterate over each chunk
-  ## TODO: maybe this can be done in parallel with bplapply() ?
-  res <- lapply(
-    chunk_names,
-    FUN = .write_chunk,
-    x = x,
-    path = path,
-    metadata = metadata
+  ## TODO: maybe this can be done in parallel with bpmapply() ?
+  res <- Map(
+    f = .write_chunk,
+    chunk_paths,
+    chunk_indices,
+    MoreArgs = list(
+      x = x,
+      metadata = metadata_v3
+    )
   )
 
   return(invisible(all(unlist(res))))
@@ -256,17 +269,13 @@ write_zarr_array <- function(
   expand.grid(lapply(n_chunks_in_dim, seq_len)) - 1
 }
 
-.write_chunk <- function(chunk_id, x, path, metadata) {
-  chunk_dim <- unlist(metadata$chunks)
-  dim_sep <- metadata$dimension_separator
-
-  chunk_id_split <- as.integer(strsplit(chunk_id, dim_sep, fixed = TRUE)[[1]])
-  chunk_path <- paste0(path, chunk_id)
+.write_chunk <- function(chunk_path, chunk_index, x, metadata) {
+  chunk_dim <- unlist(metadata$chunk_grid$configuration$chunk_shape)
 
   idx_in_array <- list()
   for (j in seq_along(dim(x))) {
     idx_in_array[[j]] <- which(
-      (seq_len(dim(x)[j]) - 1) %/% chunk_dim[j] == chunk_id_split[j]
+      (seq_len(dim(x)[j]) - 1) %/% chunk_dim[j] == chunk_index[j]
     )
   }
 
@@ -291,11 +300,6 @@ write_zarr_array <- function(
     chunk_in_mem <- temp_chunk
   }
 
-  ## check the chunk path exists, and create if not
-  if (isFALSE(dir.exists(dirname(chunk_path)))) {
-    dir.create(dirname(chunk_path), recursive = TRUE, showWarnings = FALSE)
-  }
-
   .compress_and_write_chunk(
     input_chunk = chunk_in_mem,
     chunk_path = chunk_path,
@@ -342,16 +346,47 @@ update_zarr_array <- function(zarr_array_path, x, index) {
   stopifnot(is.list(index))
 
   zarr_array_path <- .normalize_array_path(zarr_array_path)
-  metadata <- .read_array_metadata(zarr_array_path, ".zarray")
-  metadata_v3 <- .convert_metadata_version(
-    metadata,
-    version_from = 2,
-    version_to = 3
+
+  metadata_files <- setNames(
+    file.exists(paste0(zarr_array_path, c(".zarray", "zarr.json"))),
+    c(".zarray", "zarr.json")
   )
+
+  if (metadata_files[".zarray"] && metadata_files["zarr.json"]) {
+    stop(
+      "The path contains both `.zarray` (Zarr V2 specification) and ",
+      "`zarr.json` (Zarr V3 specification) metadata files.\n",
+      "An array or group must conform to either the Zarr V2 or V3 ",
+      "specification.",
+      call. = FALSE
+    )
+  }
+  if (!any(metadata_files)) {
+    stop(
+      "The path does not contain any metadata files. ",
+      "It must contain one of: ",
+      "  - `.zarray` (Zarr V2 specification)\n",
+      "  - `zarr.json` (Zarr V3 specification)",
+      call. = FALSE
+    )
+  }
+
+  metadata <- .read_array_metadata(
+    zarr_array_path,
+    names(metadata_files)[metadata_files]
+  )
+
+  if (metadata$zarr_format == 2) {
+    metadata <- .convert_metadata_version(
+      metadata,
+      version_from = 2,
+      version_to = 3
+    )
+  }
   index <- check_index(index, metadata = metadata)
 
   existing_storage <- switch(
-    metadata_v3$datatype$base_type,
+    metadata$datatype$base_type,
     "uint" = "integer",
     "int" = "integer",
     "float" = "double",
@@ -364,15 +399,25 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     stop("New data is not of the same type as the existing array.")
   }
 
+  metadata$configured_encoders <- .configure_codecs(
+    codecs = metadata$codecs,
+    operation = "encode"
+  )
+  metadata$configured_decoders <- .configure_codecs(
+    codecs = metadata$codecs,
+    operation = "decode"
+  )
+
   zarr_dim <- unlist(metadata$shape)
-  chunk_dim <- unlist(metadata$chunks)
+  chunk_dim <- unlist(metadata$chunk_grid$configuration$chunk_shape)
 
   ## coerce x to the same shape as the zarr to be updated
   x <- array(x, dim = lengths(index))
 
   ## create all possible chunk names, then remove those that won't be touched
-  chunk_names <- expand.grid(lapply(ceiling(zarr_dim / chunk_dim), seq_len)) - 1
-  chunk_needed <- rep(FALSE, nrow(chunk_names))
+  chunk_indices <- expand.grid(lapply(ceiling(zarr_dim / chunk_dim), seq_len)) -
+    1
+  chunk_needed <- rep(FALSE, nrow(chunk_indices))
 
   ## determine which chunk each of the requests indices belongs to
   chunk_idx <- .mapply(
@@ -383,66 +428,65 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     MoreArgs = NULL
   )
 
-  for (i in seq_len(nrow(chunk_names))) {
+  for (i in seq_len(nrow(chunk_indices))) {
     idx_in_zarr <- list()
     for (j in seq_along(zarr_dim)) {
-      idx_in_zarr[[j]] <- index[[j]][which(chunk_idx[[j]] == chunk_names[i, j])]
+      idx_in_zarr[[j]] <- index[[j]][which(
+        chunk_idx[[j]] == chunk_indices[i, j]
+      )]
     }
     chunk_needed[i] <- all(lengths(idx_in_zarr) > 0)
   }
-  chunk_names <- chunk_names[chunk_needed, , drop = FALSE]
+  chunk_indices <- chunk_indices[chunk_needed, , drop = FALSE]
   chunk_names <- .create_chunk_names(
-    chunk_names,
-    metadata_v3
+    chunk_indices,
+    metadata
   )
+  chunk_indices <- apply(chunk_indices, 1, identity, simplify = FALSE)
+  chunk_paths <- paste0(zarr_array_path, chunk_names)
 
   ## only update the chunks that need to be
-  ## TODO: maybe this can be done in parallel is bplapply() ?
-  res <- lapply(
-    chunk_names,
-    FUN = .update_chunk,
-    x = x,
-    path = zarr_array_path,
-    chunk_dim = chunk_dim,
-    chunk_idx = chunk_idx,
-    index = index,
-    metadata = metadata,
-    metadata_v3 = metadata_v3
+  ## TODO: maybe this can be done in parallel is bpmapply() ?
+  res <- Map(
+    f = .update_chunk,
+    chunk_paths,
+    chunk_indices,
+    MoreArgs = list(
+      x = x,
+      chunk_dim = chunk_dim,
+      chunk_idx = chunk_idx,
+      index = index,
+      metadata = metadata
+    )
   )
 
   return(invisible(all(unlist(res))))
 }
 
 .update_chunk <- function(
-  chunk_id,
+  chunk_path,
+  chunk_index,
   x,
   path,
   chunk_dim,
   chunk_idx,
   index,
   # FIXME: once we fully switch to v3, we can remove this argument
-  metadata,
-  metadata_v3
+  metadata
 ) {
-  chunk_id_split <- as.integer(
-    strsplit(chunk_id, metadata$dimension_separator, fixed = TRUE)[[1]]
-  )
-  chunk_path <- paste0(path, chunk_id)
-
   ## determine which elements of x are being used and where in this specific
   ## chunk they should be inserted
   ## TODO: This is pretty ugly, maybe there's something more elegant
   idx_in_zarr <- idx_in_x <- idx_in_chunk <- list()
   for (j in seq_along(chunk_dim)) {
-    idx_in_x[[j]] <- which(chunk_idx[[j]] == chunk_id_split[j])
+    idx_in_x[[j]] <- which(chunk_idx[[j]] == chunk_index[j])
     idx_in_zarr[[j]] <- index[[j]][idx_in_x[[j]]]
     idx_in_chunk[[j]] <- ((idx_in_zarr[[j]] - 1) %% chunk_dim[j]) + 1
   }
 
   chunk_in_mem <- read_chunk(
-    zarr_array_path = path,
-    chunk_name = chunk_id,
-    metadata = metadata_v3,
+    chunk_path = chunk_path,
+    metadata = metadata,
     fill = TRUE
   )
 
@@ -459,8 +503,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
   .compress_and_write_chunk(
     input_chunk = chunk_in_mem,
     chunk_path = chunk_path,
-    metadata,
-    is_base64 = (metadata$datatype$base_type == "unicode")
+    metadata
   )
 }
 
@@ -470,10 +513,6 @@ update_zarr_array <- function(zarr_array_path, x, index) {
 #'   converted to a raw vector before compression.
 #' @param chunk_path Character string giving the path to the chunk that should
 #'   be written.
-#' @param is_base64 When dealing with Py_unicode strings we convert them to
-#' base64 strings for storage in our intermediate R arrays.  This argument
-#' indicates if base64 is in use, because the conversion to raw in .as_raw
-#' should be done differently for base64 strings vs other types.
 #'
 #' @returns Returns `TRUE` if writing is successful.  Mostly called for the
 #'   side-effect of writing the compressed chunk to disk.
@@ -482,45 +521,36 @@ update_zarr_array <- function(zarr_array_path, x, index) {
 .compress_and_write_chunk <- function(
   input_chunk,
   chunk_path,
-  metadata,
-  is_base64 = FALSE
+  metadata
 ) {
+  ## check the chunk path exists, and create if not
+  if (isFALSE(dir.exists(dirname(chunk_path)))) {
+    dir.create(dirname(chunk_path), recursive = TRUE, showWarnings = FALSE)
+  }
+
   # Array to array codecs
-  if (metadata$order == "C") {
-    input_chunk <- codec_transpose_encode(
-      input_chunk,
-      indices = rev(seq_along(dim(input_chunk)))
-    )
+  for (codec in metadata$configured_encoders[["array_array"]]) {
+    input_chunk <- do.call(codec, list(input_chunk))
   }
-
   # Array to bytes codecs
-  ## convert strings to Unicode if required
-  if (metadata$datatype$base_type == "unicode") {
-    input_chunk <- .unicode_to_int(
-      input = input_chunk,
-      typestr = metadata$dtype
-    )
-  }
-
-  ## the compression tools need a raw vector
-  raw_chunk <- .as_raw(
-    as.vector(input_chunk),
-    nchar = metadata$datatype$nbytes,
-    is_base64 = is_base64
-  )
-
-  # Endianness in unicode is handled during the conversion to/from
-  if (metadata$datatype$base_type != "unicode") {
-    raw_chunk <- codec_bytes_encode(
-      raw_chunk,
-      endian = metadata$datatype$endian,
-      metadata$datatype$nbytes
+  for (codec in metadata$configured_encoders[["array_bytes"]]) {
+    endian <- metadata$codecs[["bytes"]]$configuration %||% NA_character_
+    raw_chunk <- do.call(
+      codec,
+      list(as.vector(input_chunk), metadata$datatype, endian)
     )
   }
 
   # Bytes to bytes codecs
-  compressor <- metadata$compressor
-  if (is.null(compressor)) {
+  codecs <- metadata$codecs
+  compressor <- NULL
+  compressor$id <- names(codecs)[match(
+    TRUE,
+    names(codecs) %in% c("zstd", "blosc", "gzip", "zlib", "bz2", "lzma", "lz4")
+  )]
+  compressor$level <- unlist(codecs[[compressor$id]]$configuration)
+
+  if (is.na(compressor$id)) {
     compressed_chunk <- raw_chunk
   } else if (compressor$id == "blosc") {
     compressed_chunk <- .Call(
@@ -546,7 +576,7 @@ update_zarr_array <- function(zarr_array_path, x, index) {
     ## compression. We should do that too for compatibility
     ## TODO: probably faster to do this in C and avoid copying the vector
     compressed_chunk <- c(
-      .as_raw(length(raw_chunk), nchar = 4),
+      writeBin(length(raw_chunk), raw(), size = 4, endian = "little"),
       compressed_chunk
     )
   } else if (compressor$id == "zstd") {
@@ -556,11 +586,9 @@ update_zarr_array <- function(zarr_array_path, x, index) {
       as.integer(compressor$level),
       PACKAGE = "Rarr"
     )
-  } else {
-    stop("Unsupported compression tool")
   }
 
-  if (!is.null(compressor) && compressor$id %in% c("gzip", "bz2", "lzma")) {
+  if (compressor$id %in% c("gzip", "bz2", "lzma")) {
     writeBin(raw_chunk, con = con, useBytes = TRUE)
   } else {
     writeBin(compressed_chunk, con = chunk_path)
@@ -569,38 +597,16 @@ update_zarr_array <- function(zarr_array_path, x, index) {
   return(invisible(TRUE))
 }
 
-.as_raw <- function(d, nchar, is_base64) {
-  ## we need to create fixed length strings either via padding or trimming
-  if (is.character(d)) {
-    if (is_base64) {
-      raw_list <- lapply(d, jsonlite::base64_dec)
-    } else {
-      raw_list <- iconv(d, toRaw = TRUE)
-    }
-    unlist(
-      lapply(
-        raw_list,
-        FUN = function(x, nchar) {
-          if (!is.null(x)) {
-            length(x) <- nchar
-          }
-          return(x)
-        },
-        nchar
-      )
-    )
-  } else {
-    writeBin(d, raw(), size = nchar, endian = "little")
-  }
-}
-
 .check_chunk_shape <- function(x_dim, chunk_dim) {
   if (length(x_dim) != length(chunk_dim)) {
     stop("The dimensions of the chunk must equal the dimensions of the array.")
   }
 
   for (i in seq_along(x_dim)) {
-    if ((x_dim[i] < chunk_dim[i]) || (chunk_dim[i] < 1)) {
+    # spec says:
+    # "The chunk shape elements are non-zero when the corresponding dimensions
+    # of the arrays have non-zero length."
+    if ((x_dim[i] < chunk_dim[i]) || (chunk_dim[i] == 0 && x_dim[i] != 0)) {
       stop("Chunk dimensions outside the extent of the array")
     }
   }
