@@ -15,8 +15,9 @@
   }
 
   zarray <- list(
-    shape = array_shape,
-    chunks = chunk_shape,
+    # the spec states these need to be json arrays, so we need to avoid auto_unboxing
+    shape = as.list(array_shape),
+    chunks = as.list(chunk_shape),
     dtype = data_type,
     fill_value = fill_value,
     dimension_separator = dimension_separator,
