@@ -55,3 +55,25 @@ test_that("i16 zarr array can be written", {
     roundtrip_i16_array
   )
 })
+
+test_that("v2 and v3 return identical results", {
+  zarr_v2 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "int16.zarr",
+    package = "Rarr"
+  )
+  zarr_v3 <- system.file(
+    "extdata",
+    "zarr_examples",
+    "column-first",
+    "int16_v3.zarr",
+    package = "Rarr"
+  )
+
+  expect_no_condition(i16_v2 <- read_zarr_array(zarr_v2))
+  expect_no_condition(i16_v3 <- read_zarr_array(zarr_v3))
+
+  expect_identical(i16_v2, i16_v3)
+})
